use Renard::Incunabula::Common::Setup;
package Renard::Incunabula::Frontend::Gtk3;
# ABSTRACT: Provides helpers for dealing with Gtk3 code
$Renard::Incunabula::Frontend::Gtk3::VERSION = '0.004';
1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Renard::Incunabula::Frontend::Gtk3 - Provides helpers for dealing with Gtk3 code

=head1 VERSION

version 0.004

=head1 SEE ALSO

L<Repository information|http://project-renard.github.io/doc/development/repo/p5-Renard-Incunabula-Frontend-Gtk3/>

=head1 AUTHOR

Project Renard

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2017 by Project Renard.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
