theme = ( core + pbp + security + maintenance ) * bugs
include = CodeLayout::ProhibitSpaceIndentation CodeLayout::ProhibitTrailingWhitespace CodeLayout::RequireConsistentNewlines CodeLayout::TabIndentSpaceAlign

[TestingAndDebugging::RequireUseStrict]
equivalent_modules = Modern::Perl Moo Moo::Role Renard::Incunabula::Common::Setup

[TestingAndDebugging::RequireUseWarnings]
equivalent_modules = Modern::Perl Moo Moo::Role Renard::Incunabula::Common::Setup

# vim: ft=dosini
